/*
 *
 */

#include <stdint.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <getopt.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <linux/types.h>
#include <linux/spi/spidev.h>
#include <string.h>

#include "rp_spi.h" /*struct rpi_in */

//#define ARRAY_SIZE(a) (sizeof(a) / sizeof((a)[0]))
#define ARRAY_SIZE 16
static void pabort(const char *s)
{
	perror(s);
	abort();
}

static const char *device = "/dev/spidev0.1";
static uint8_t mode = 0;
static uint8_t bits = 8;
static uint32_t speed = 500000;
static uint16_t delay = 0;
static int fd;

/**
 * \brief Testovaci funkce. Odesle same nuly. Delka 64bit. Vysledek vypise.
 */
void transfer()
{
	int ret;
	uint8_t tx[]={0,0,0,0,0,0,0,0} ;/*= {
		0x00, 0x00, 0x01, 0x23, 0x45, 0x67,
		0x89, 0xAB, 0xCD, 0xEF, 0x01, 0x23,
		0x45, 0x67, 0x89, 0xAB, 0xCD, 0xEF,
		0x01, 0x23, 0x45, 0x67, 0x89, 0xAB,
		0xCD, 0xEF
	};/**/
	printf("ARRAY_SIZE=%d\n",ARRAY_SIZE);
	uint8_t rx[ARRAY_SIZE] = {0, }; /* inicializace vsech prvku na nulu? */
	struct spi_ioc_transfer tr = {
		.tx_buf = (unsigned long)tx,
		.rx_buf = (unsigned long)rx,
		.len = ARRAY_SIZE,
		.delay_usecs = delay,
		.speed_hz = speed,
		.bits_per_word = bits,
	};

	ret = ioctl(fd, SPI_IOC_MESSAGE(1), &tr);
	if (ret < 1)
		pabort("can't send spi message");

	for (ret = 0; ret < ARRAY_SIZE; ret++) {
		if (!(ret % 6))
			puts("");
		printf("%.2X ", rx[ret]);
	}
	puts("");
}
struct rpi_in spi_read(uint8_t * tx)
{
	uint8_t *uint8_p;
	uint16_t tmp;
	int sign;
	struct rpi_in in;
	int ret;

	uint8_t rx[ARRAY_SIZE] = {0, }; /* inicializace vsech prvku na nulu? */
	struct spi_ioc_transfer tr = {
		.tx_buf = (unsigned long)tx,
		.rx_buf = (unsigned long)rx,
		.len = ARRAY_SIZE,
		.delay_usecs = delay,
		.speed_hz = speed,
		.bits_per_word = bits,
	};

	memset(&in,0,sizeof(in));

	ret = ioctl(fd, SPI_IOC_MESSAGE(1), &tr);
	/*vypisovani prichozich dat */

	if (ret < 1)
		pabort("can't send spi message");

	for (ret = 0; ret < ARRAY_SIZE; ret++) {
		in.debug_rx[ret]=rx[ret];
	}

	/*/

	/*prichozi data:
	 * rx[0] - bity 127 downto 120 the first income bit..127
	 * rx[1] - bity 119 downto 112
	 * rx[2] - bity 111 downto 104
	 * rx[3] - bity 103 downto 96
	 * rx[4] - bity 95 downto 88
	 * rx[5] - bity 87 downto 80
	 * rx[6] - bity 79 downto 72
	 * rx[7] - bity 71 downto 64
	 * rx[8] - bity 63 downto 56
	 * rx[9] - bity 55 downto 48
	 * rx[10] - bity 47 downto 40
	 * rx[11] - bity 39 downto 32
	 * rx[12] - bity 31 downto 24
	 * rx[13] - bity 23 downto 16
	 * rx[14] - bity 15 downto 8
	 * rx[15] - bity 7 downto 0	the last income bit..0
	 */

	/*uprava endianity pozice*/
	uint8_p=(uint8_t*)&in.pozice_raw;
	/* x86 je Little-Endian */
	uint8_p[0]=rx[3]; /* LSB */
	uint8_p[1]=rx[2];
	uint8_p[2]=rx[1];
	uint8_p[3]=rx[0]; /*MSB*/ /*with sign bit*/

	/*halove sondy
	 * hal1 - bit95
	 * hal2 - bit94
	 * hal3 - bit93
	 */
	in.hal1=!!(0x80 & rx[4]);
	in.hal2=!!(0x40 & rx[4]);
	in.hal3=!!(0x20 & rx[4]);

	/* index position
	 * bits 92 downto 81
	 * 	92..88 in rx[4] last 5 bits (from left)
	 * 	87..81 in rx[5] first 7 bits (from left)
	 */
	in.index_position=0x1F & rx[4];
	in.index_position<<=8;
	in.index_position|=0xFE & rx[5];
	in.index_position>>=1;

	/* current measurments count
	 * bits 80 downto 72
	 * bit 80 in rx[5]
	 * bits 79..72 in rx[6]
	 */

	in.adc_m_count=0x01 & rx[5];
	in.adc_m_count<<=8;
	in.adc_m_count|=rx[6];


	/** currents
	 * ch2 - bits 71 downto 48
	 * 	71..64 in rx[7] - all byte
	 * 	63..56 in rx[8] - all byte
	 * 	55..48 in rx[9] - all byte
	 * ch0 - bits 47 downto 24
	 * 	47..40 in rx[10] - all byte
	 * 	39..32 in rx[11] - all byte
	 * 	31..24 in rx[12] - all byte
	 * ch1 - bits 23 downto 0
	 * 	23..16 in rx[13] - all byte
	 * 	15..8 in rx[14] - all byte
	 * 	7..0 in rx[15] - all byte
	 */

	in.ch2=rx[7];
	in.ch2<<=8;
	in.ch2|=rx[8];
	in.ch2<<=8;
	in.ch2|=rx[9];

	in.ch0=rx[10];
	in.ch0<<=8;
	in.ch0|=rx[11];
	in.ch0<<=8;
	in.ch0|=rx[12];

	in.ch1=rx[13];
	in.ch1<<=8;
	in.ch1|=rx[14];
	in.ch1<<=8;
	in.ch1|=rx[15];


	return in;


}

int spi_init()
{
	int ret = 0;

	fd = open(device, O_RDWR);
	if (fd < 0)
		pabort("can't open device");
	printf("device open\n");
	/*
	 * spi mode
	 */
	ret = ioctl(fd, SPI_IOC_WR_MODE, &mode);
	if (ret == -1)
		pabort("can't set spi mode");

	ret = ioctl(fd, SPI_IOC_RD_MODE, &mode);
	if (ret == -1)
		pabort("can't get spi mode");

	/*
	 * bits per word
	 */
	ret = ioctl(fd, SPI_IOC_WR_BITS_PER_WORD, &bits);
	if (ret == -1)
		pabort("can't set bits per word");

	ret = ioctl(fd, SPI_IOC_RD_BITS_PER_WORD, &bits);
	if (ret == -1)
		pabort("can't get bits per word");

	/*
	 * max speed hz
	 */
	ret = ioctl(fd, SPI_IOC_WR_MAX_SPEED_HZ, &speed);
	if (ret == -1)
		pabort("can't set max speed hz");

	ret = ioctl(fd, SPI_IOC_RD_MAX_SPEED_HZ, &speed);
	if (ret == -1)
		pabort("can't get max speed hz");

	printf("spi mode: %d\n", mode);
	printf("bits per word: %d\n", bits);
	printf("delay: %d\n", delay);
	printf("max speed: %d Hz (%d KHz)\n", speed, speed/1000);


	return ret;
}

void spi_disable(){
	close(fd);

}
